/**************************************************************************
*
* Filename:    PS4000con.c
*
* Copyright:   Pico Technology Limited 2010
*
* Author:      PT
*
* Description:
*   This is a console-mode program that demonstrates how to use the
*   PS4000 driver.
*
* Examples:
*    Collect a block of samples immediately
*    Collect a block of samples when a trigger event occurs
*    Collect a stream of data immediately
*    Collect a stream of data when a trigger event occurs
*    Set Signal Generator, using built in or custom signals
*
*	To build this application
*		Windows: set up a project for a 32-bit console mode application
*			 add this file to the project
*			 add PS4000.lib to the project (Microsoft C only)
*			 add ps4000Api.h and picoStatus.h to the project
*			 then build the project
*
*		Linux: gcc -lps4000 PS4000con.c -oPS4000con
*		       ./PS4000con
*
*
***************************************************************************/

//#ifdef WIN32
/* Headers for Windows */
#include "windows.h"
#include <conio.h>
#include <stdio.h>

/* Definitions of PS4000 driver routines on Windows*/
#include "ps4000Api.h"

#define PREF4 __stdcall



int cycles = 0;

#define BUFFER_SIZE 	1024
#define MAX_CHANNELS 4
#define QUAD_SCOPE 4
#define TRIPLE_SCOPE 3
#define DUAL_SCOPE 2
#define SEGMEM		10

typedef struct
{
	short DCcoupled;
	short range;
	short enabled;
}CHANNEL_SETTINGS;

typedef enum
{
	MODEL_NONE = 0,
	MODEL_PS4223 = 4223,
	MODEL_PS4224 = 4224,
	MODEL_PS4423 = 4423,
	MODEL_PS4424 = 4424,
	MODEL_PS4226 = 4226, 
	MODEL_PS4227 = 4227,
	MODEL_PS4262 = 4262
} MODEL_TYPE;

typedef struct tTriggerDirections
{
	enum enThresholdDirection channelA;
	enum enThresholdDirection channelB;
	enum enThresholdDirection channelC;
	enum enThresholdDirection channelD;
	enum enThresholdDirection ext;
	enum enThresholdDirection aux;
}TRIGGER_DIRECTIONS;

typedef struct tPwq
{
	struct tPwqConditions * conditions;
	short nConditions;
	enum enThresholdDirection direction;
	unsigned long lower;
	unsigned long upper;
	enum enPulseWidthType type;
}PWQ;

typedef struct
{
	short							handle;
	MODEL_TYPE				model;
	PS4000_RANGE			firstRange;
	PS4000_RANGE			lastRange;
	unsigned char			signalGenerator;
	unsigned char			ETS;
	short							ChannelCount;
	CHANNEL_SETTINGS	channelSettings[MAX_CHANNELS];
	PS4000_RANGE			triggerRange;
}UNIT_MODEL;

unsigned long			timebase = 8;
short							oversample = 1;
BOOL							scaleVoltages = TRUE;

unsigned short inputRanges [PS4000_MAX_RANGES] = { 
10,
20,
50,
100,
200,
500,
1000,
2000,
5000,
10000,
20000,
50000};

short     		g_ready = FALSE;
long long 		g_times [PS4000_MAX_CHANNELS];
short     		g_timeUnit;
long      		g_sampleCount;
unsigned long g_startIndex;
short         g_autoStop;

short			g_trig = 0;
unsigned long	g_trigAt = 0;

/****************************************************************************
* Callback
* used by PS4000 data streaimng collection calls, on receipt of data.
* used to set global flags etc checked by user routines
****************************************************************************/
void PREF4 CallBackStreaming
(
 short handle,
 long noOfSamples,
 unsigned long startIndex,
 short overflow,
 unsigned long triggerAt,
 short triggered,
 short autoStop,
 void * pParameter
 )
{
	// used for streaming
	g_sampleCount = noOfSamples;
	g_startIndex = startIndex;
	g_autoStop = autoStop;

	// flag to say done reading data
	g_ready = TRUE;

	// flags to show if & where a trigger has occurred
	g_trig = triggered;
	g_trigAt = triggerAt;
}

/****************************************************************************
* Callback
* used by PS4000 data block collection calls, on receipt of data.
* used to set global flags etc checked by user routines
****************************************************************************/
void PREF4 CallBackBlock
(
 short handle,
 PICO_STATUS status,
 void * pParameter
 )
{
	// flag to say done reading data
	g_ready = TRUE;
}

/****************************************************************************
* SetDefaults - restore default settings
****************************************************************************/
void SetDefaults(UNIT_MODEL * unit)
{
	PICO_STATUS status;
	int i;

	if(unit->ETS)
	{
		status = ps4000SetEts(unit->handle, PS4000_ETS_OFF, 0, 0, NULL); // Turn off ETS
		printf(status?"SetDefaults: ps4000SetEts ------ %d \n":"", status);
	}

	for (i = 0; i < unit->ChannelCount; i++) // reset channels to most recent settings
	{
		status = ps4000SetChannel(unit->handle, PS4000_CHANNEL_A + i,
															unit->channelSettings[PS4000_CHANNEL_A + i].enabled,
															unit->channelSettings[PS4000_CHANNEL_A + i].DCcoupled,
															unit->channelSettings[PS4000_CHANNEL_A + i].range);

		printf(status?"SetDefaults: ps4000SetChannel(channel: %d)------ %d \n":"", i, status);
	}
}

/****************************************************************************
* adc_to_mv
*
* Convert an 16-bit ADC count into millivolts
****************************************************************************/

int adc_to_mv(long raw, int ch)
{
	return (raw * inputRanges[ch]) / PS4000_MAX_VALUE;
}

/****************************************************************************
* mv_to_adc
*
* Convert a millivolt value into a 16-bit ADC count
*
*  (useful for setting trigger thresholds)
****************************************************************************/
short mv_to_adc(short mv, short ch)
{
	return ((mv * PS4000_MAX_VALUE ) / inputRanges[ch]);
}

/****************************************************************************
* RapidBlockDataHandler
* - Used by Rapid block data routines
* - acquires data (user sets trigger mode before calling), displays 10 items
*   and saves all to data.txt
* Input :
* - unit : the unit to use.
* - text : the text to display before the display of data slice
* - offset : the offset into the data buffer to start the display's slice.
****************************************************************************/
int RapidBlockDataHandler(UNIT_MODEL * unit, char * text, int offset)
{
	int i, j;
	long timeInterval;
	unsigned long sampleCount = 50000;
	FILE * fp = NULL;
	long maxSamples;
	short * buffers[PS4000_MAX_CHANNEL_BUFFERS * 2];
	long timeIndisposed;
	long nMaxSamples;
	unsigned long segmentIndex;
	unsigned long noOfSamples;
	PICO_STATUS status;

/*  find the maximum number of samples, the time interval (in timeUnits),
	*		 the most suitable time units, and the maximum oversample at the current timebase*/
	while (ps4000GetTimebase(	unit->handle, timebase, sampleCount, &timeInterval, oversample,	&maxSamples, 0))
	{
		timebase++;
	}
	printf("Rapid Block mode with aggregation:- timebase: %lu\toversample:%hd\n", timebase, oversample);

	// Set the memory segments (must be euqal or more than no of waveforms)
	ps4000MemorySegments(unit->handle, 100, &nMaxSamples);

	// smapleCount must be < nMaxSamples
	sampleCount = 20000;
	printf("Rapid Block Mode with aggregation: memory Max smaples = %ld \n", nMaxSamples);

	// Set the number of waveforms to 100
	ps4000SetNoOfCaptures(unit->handle, 100);

	/* Start it collecting, then wait for completion*/
	g_ready = FALSE;

	status = ps4000RunBlock(unit->handle, 0, sampleCount, timebase, oversample,
													&timeIndisposed, 0, CallBackBlock, NULL);

	printf("RapidBlockDataHandler::Run Block : %i\n", status);
	printf("Waiting for trigger...Press a key to abort\n");

	while (!g_ready && !_kbhit())
	{
		Sleep(0);
	}

	for (i = 0; i < unit->ChannelCount; i++) 
	{
		buffers[i * 2] = (short*)malloc(sampleCount * sizeof(short));
		buffers[i * 2 + 1] = (short*)malloc(sampleCount * sizeof(short));
		ps4000SetDataBuffers(unit->handle, (PS4000_CHANNEL)i, buffers[i * 2], buffers[i* 2 + 1], sampleCount);
	}

	ps4000Stop(unit->handle);

	if (g_ready) 
	{
		// retrieve data of 10 Segments 
		// one segment is a bulk of data of one waveform
		// and printf out the first 10 readings
		fopen_s(&fp, "data.txt", "w");

		noOfSamples = sampleCount;

		for (segmentIndex = 80; segmentIndex < 90; segmentIndex ++)
		{
			printf("\nRapid Block Mode with aggregation: Reading Segement:-- %lu \n", segmentIndex);
			// Get values of 
			sampleCount = noOfSamples;
			status = ps4000GetValues(unit->handle, 0, (unsigned long*) &sampleCount, 1, RATIO_MODE_NONE, segmentIndex, NULL);
			printf("\nRapid Block Mode with aggregation: Reading Segement:-- ps4000GetValues: %i \n", status);
			/* Print out the first 10 readings, converting the readings to mV if required */
			printf(text);
			printf("Value (%s)\n", ( scaleVoltages ) ? ("mV") : ("ADC Counts"));

			for (j = 0; j < unit->ChannelCount; j++) 
			{
				if (unit->channelSettings[j].enabled) 
					printf("  Ch%c:      ", 'A' + j);
			}
			printf("\n");

			for (i = offset; i < offset+10; i++) 
			{
				for (j = 0; j < unit->ChannelCount; j++) 
				{
					if (unit->channelSettings[j].enabled) 
					{
						printf("%6d      ",scaleVoltages?
								adc_to_mv(buffers[j * 2][i],unit->channelSettings[PS4000_CHANNEL_A + j].range)
								: buffers[j * 2][i]);
					}
				}
				printf("\n");
			}

			sampleCount = __min(sampleCount, BUFFER_SIZE);
			if (fp != NULL)
			{
				fprintf(fp, "Rapid Block mode with aggregation Data log\n\n");
				fprintf(fp,"Results shown for each of the %d Channels are......\n",unit->ChannelCount);
				fprintf(fp,"Maximum Aggregated value ADC Count & mV, Minimum Aggregated value ADC Count & mV\n\n");

				fprintf(fp, "Time  ");
				for (i = 0; i < unit->ChannelCount; i++)
				{
					if (unit->channelSettings[i].enabled) 
						fprintf(fp,"Ch%C   Max ADC   Max mV   Min ADC   Min mV   ", 'A' + i);
				}
				fprintf(fp, "\n");

				for (i = 0; i < sampleCount; i++) 
				{
					fprintf(fp, "%5lld ", g_times[0] + (long long)(i * timeInterval));
					for (j = 0; j < unit->ChannelCount; j++) 
					{
						
						if (unit->channelSettings[j].enabled) 
						{
							fprintf(	fp,
							"Ch%C  %6d = %+6dmV, %6d = %+6dmV   ",
							'A' + j,
							buffers[j * 2][i],
							adc_to_mv(buffers[j * 2][i], unit->channelSettings[PS4000_CHANNEL_A + j].range),
							buffers[j * 2 + 1][i],
							adc_to_mv(buffers[j * 2 + 1][i], unit->channelSettings[PS4000_CHANNEL_A + j].range));
						}
					}
					fprintf(fp, "\n");
				}
			}
			else
				printf("Cannot open the file data.txt for writing. \nPlease ensure that you have permission to access. \n");
		}
	} 
	else 
	{
		printf("data collection aborted\n");
		_getch();
	}

	if (fp != NULL)
		fclose(fp);

	for (i = 0; i < unit->ChannelCount * 2; i++) 
	{
		free(buffers[i]);
	}
	return 1;
}

/****************************************************************************
* No_Agg_RapidBlockDataHandler
* - Used by Rapid block data routines
* - acquires data (user sets trigger mode before calling), displays 10 items
*   and saves all to data.txt
* Input :
* - unit : the unit to use.
* - text : the text to display before the display of data slice
* - offset : the offset into the data buffer to start the display's slice.
****************************************************************************/
int No_Agg_RapidBlockDataHandler(UNIT_MODEL * unit, char * text, int offset)
{
	int i, j;
	long timeInterval;
	long sampleCount = 50000;
	FILE * fp = NULL;
	long maxSamples;
	short * buffers[PS4000_MAX_CHANNEL_BUFFERS][SEGMEM];
	short overflow[SEGMEM] = {1,1,1,1,1,1,1,1,1,1};
	long timeIndisposed;
	long nMaxSamples;
	unsigned long segmentIndex;
	unsigned long noOfSamples;
	PICO_STATUS status;


	/*  find the maximum number of samples, the time interval (in timeUnits),
	*		 the most suitable time units, and the maximum oversample at the current timebase*/
	while (ps4000GetTimebase(unit->handle, timebase, sampleCount,	&timeInterval, oversample, &maxSamples, 0))
	{
		timebase++;
	}
	printf("Rapid Block mode without aggregation:- timebase: %lu\toversample:%hd\n", timebase, oversample);


	// Set the memory segments (must be euqal or more than no of waveforms)
	ps4000MemorySegments(unit->handle, 100, &nMaxSamples);

	// smapleCount must be < nMaxSamples
	sampleCount = 50000;
	printf("Rapid Block Mode without aggregation: memory Max samples = %ld \n", nMaxSamples);

	// Set the number of waveforms to 100
	ps4000SetNoOfCaptures(unit->handle, 100);

	/* Start it collecting, then wait for completion*/
	g_ready = FALSE;

	
	status = ps4000RunBlock(unit->handle, 0, sampleCount, timebase, oversample, &timeIndisposed, 0, CallBackBlock, NULL);
	/*printf("Run Block : %i", status);*/
	printf("Waiting for trigger...Press a key to abort\n");

	while (!g_ready && !_kbhit())
	{
		Sleep(0);
	}

	// set the data buffer bulk
	// (SEGMEM)10 segments for each channel
	for (i = 0; i < unit->ChannelCount; i++) 
	{
		for (j = 0; j < SEGMEM; j ++)
		{
			buffers[i][j] = (short*)malloc(sampleCount * sizeof(short));
			ps4000SetDataBufferBulk(unit->handle, (PS4000_CHANNEL)i, buffers[i][j], sampleCount, j);
		}
	}

	ps4000Stop(unit->handle);

	if (g_ready) 
	{
		// retrieve data of 10 Segments 
		// one segment is a bulk of data of one waveform
		// and printf out the first 10 readings
		fopen_s(&fp, "data.txt", "w");

		noOfSamples = sampleCount;

		ps4000GetValuesBulk(unit->handle, (long unsigned *) &sampleCount, 0// start segment index
			, 9 // end segment index (Please refer to Programmer's Guide for more details)
			, overflow);

		for (segmentIndex = 0; segmentIndex < 9; segmentIndex ++)
		{
			printf("\nRapid Block Mode without aggregation: Reading Segement:-- %lu \n", segmentIndex);
			// Get values of 
			sampleCount = noOfSamples;
			ps4000GetValues(unit->handle, 0, (unsigned long*) &sampleCount, 1, RATIO_MODE_NONE, segmentIndex, NULL);

			// Print out the first 10 readings, converting the readings to mV if required 
			printf(text);
			printf("Value (%s)\n", (scaleVoltages) ? ("mV") : ("ADC Counts"));

			for (j = 0; j < unit->ChannelCount; j++) 
			{
				if (unit->channelSettings[j].enabled) 
					printf("  Ch%c:      ", 'A' + j);
			}
			printf("\n");

			for (i = offset; i < offset+10; i++) 
			{
				for (j = 0; j < unit->ChannelCount; j++) 
				{
					if (unit->channelSettings[j].enabled) 
					{
						printf("%6d      ",scaleVoltages?
							adc_to_mv(buffers[j][segmentIndex][i],unit->channelSettings[PS4000_CHANNEL_A + j].range)
							: buffers[j][segmentIndex][i]);
					}
				}
				printf("\n");
			}

			sampleCount = __min(sampleCount, BUFFER_SIZE);
			if (fp != NULL)
			{
				fprintf(fp, "Rapid Block mode without aggregation Data log\n\n");
				fprintf(fp,"Results shown for each of the %d Channels are......\n",unit->ChannelCount);
				fprintf(fp,"ADC Count & mV\n\n");

				fprintf(fp, "Time  ");
				for (i = 0; i < unit->ChannelCount; i++)
				{
					if (unit->channelSettings[i].enabled) 
						fprintf(fp,"Ch%C     ADC         mV   ", 'A' + i);
				}
				fprintf(fp, "\n");

				for (i = 0; i < sampleCount; i++) 
				{
					fprintf(fp, "%5lld ", g_times[0] + (long long)(i * timeInterval));
					for (j = 0; j < unit->ChannelCount; j++) 
					{
						
						if (unit->channelSettings[j].enabled) 
						{
							fprintf(	fp,
							"Ch%C  %6d = %+6dmV   ",
							'A' + j,
							buffers[j][segmentIndex][i], 
							adc_to_mv(buffers[j][segmentIndex][i], 
							unit->channelSettings[PS4000_CHANNEL_A + j].range));
						}
					}
					fprintf(fp, "\n");
				}
			}
			else
				printf("Cannot open the file data.txt for writing. \nPlease ensure that you have permission to access. \n");
		}
	} 
	else 
	{
		printf("data collection aborted\n");
		_getch();
	}

	if (fp != NULL)
		fclose(fp);

	for (i = 0; i < unit->ChannelCount; i++) 
	{
		for (j = 0; j < SEGMEM; j ++)
			free(buffers[i][j]);		
	}
	return 1;
}


/****************************************************************************
* BlockDataHandler
* - Used by all block data routines
* - acquires data (user sets trigger mode before calling), displays 10 items
*   and saves all to data.txt
* Input :
* - unit : the unit to use.
* - text : the text to display before the display of data slice
* - offset : the offset into the data buffer to start the display's slice.
****************************************************************************/
void BlockDataHandler(UNIT_MODEL * unit, char * text, int offset)
{
	int i, j;
	long timeInterval;
	long sampleCount= BUFFER_SIZE;
	FILE * fp = NULL;
	long maxSamples;
	short * buffers[PS4000_MAX_CHANNEL_BUFFERS * 2];
	long timeIndisposed;
	PICO_STATUS status;

	for (i = 0; i < unit->ChannelCount; i++) 
	{
		buffers[i * 2] = (short*)malloc(sampleCount * sizeof(short));
		buffers[i * 2 + 1] = (short*)malloc(sampleCount * sizeof(short));
		status = ps4000SetDataBuffers(unit->handle, (PS4000_CHANNEL)i, buffers[i * 2], buffers[i * 2 + 1], sampleCount);

		printf("BlockDataHandler:ps4000SetDataBuffers(channel %d) ------ %d \n", i, status);
	}

	/*  find the maximum number of samples, the time interval (in timeUnits),
	*		 the most suitable time units, and the maximum oversample at the current timebase*/
	while (ps4000GetTimebase(unit->handle, timebase, sampleCount, &timeInterval, oversample, &maxSamples, 0))
	{
		timebase++;
	}

	printf("timebase: %ld\toversample:%hd\n", timebase, oversample);

	/* Start it collecting, then wait for completion*/
	g_ready = FALSE;
	status = ps4000RunBlock(unit->handle, 0, sampleCount, timebase, oversample,	&timeIndisposed, 0, CallBackBlock, NULL);

	printf("BlockDataHandler:ps4000RunBlock ------ %i \n", status);
	printf("Waiting for trigger...Press a key to abort\n");

	while (!g_ready && !_kbhit())
	{
		Sleep(0);
	}

	if (g_ready) 
	{
		status = ps4000GetValues(unit->handle, 0, (unsigned long*) &sampleCount, 1, RATIO_MODE_NONE, 0, NULL);
		printf("BlockDataHandler:ps4000GetValues ------ %i \n", status);

		/* Print out the first 10 readings, converting the readings to mV if required */
		printf(text);
		printf("Value (%s)\n", ( scaleVoltages ) ? ("mV") : ("ADC Counts"));

		for (j = 0; j < unit->ChannelCount; j++) 
		{
			if (unit->channelSettings[j].enabled) 
				printf("Channel%c:    ", 'A' + j);
		}
		printf("\n");

		for (i = offset; i < offset+10; i++) 
		{
			for (j = 0; j < unit->ChannelCount; j++) 
			{
				if (unit->channelSettings[j].enabled) 
				{
					printf("  %6d     ", scaleVoltages ? 
							adc_to_mv(buffers[j * 2][i], unit->channelSettings[PS4000_CHANNEL_A + j].range)	// If scaleVoltages, print mV value
							: buffers[j * 2][i]);																	// else print ADC Count
				}
			}
			printf("\n");
		}

		sampleCount = __min(sampleCount, BUFFER_SIZE);

		fopen_s(&fp, "block.txt", "w");
		if (fp != NULL)
		{
			fprintf(fp, "Block Data log\n\n");
			fprintf(fp,"Results shown for each of the %d Channels are......\n",unit->ChannelCount);
			fprintf(fp,"Maximum Aggregated value ADC Count & mV, Minimum Aggregated value ADC Count & mV\n\n");

			fprintf(fp, "Time  ");
			for (i = 0; i < unit->ChannelCount; i++)
			{
				if (unit->channelSettings[i].enabled) 
					fprintf(fp,"Ch%C   Max ADC   Max mV   Min ADC   Min mV   ", 'A' + i);
			}
			fprintf(fp, "\n");

			for (i = 0; i < sampleCount; i++) 
			{
				fprintf(fp, "%5lld ", g_times[0] + (long long)(i * timeInterval));
				for (j = 0; j < unit->ChannelCount; j++) 
				{
					if (unit->channelSettings[j].enabled) 
					{
						fprintf(	fp,
							"Ch%C  %6d = %+6dmV, %6d = %+6dmV   ",
							'A' + j,
							buffers[j * 2][i],
							adc_to_mv(buffers[j * 2][i], unit->channelSettings[PS4000_CHANNEL_A + j].range),
							buffers[j * 2 + 1][i],
							adc_to_mv(buffers[j * 2 + 1][i], unit->channelSettings[PS4000_CHANNEL_A + j].range));
					}
				}
				fprintf(fp, "\n");
			}
		}
		else
		{
			printf(	"Cannot open the file block.txt for writing.\n"
			"Please ensure that you have permission to access.\n");
		}
	}
	else 
	{
		printf("data collection aborted\n");
		_getch();
	}

	if ((status = ps4000Stop(unit->handle)) != PICO_OK)
		printf("BlockDataHandler:ps4000Stop ------ 0x%08lx \n", status);

	if (fp != NULL)
		fclose(fp);

	for (i = 0; i < unit->ChannelCount * 2; i++) 
	{
		free(buffers[i]);
	}
}


/****************************************************************************
* Stream Data Handler
* - Used by the two stream data examples - untriggered and triggered
* Inputs:
* - unit - the unit to sample on
* - preTrigger - the number of samples in the pre-trigger phase 
*					(0 if no trigger has been set)
***************************************************************************/
void StreamDataHandler(UNIT_MODEL * unit, unsigned long preTrigger)
{
	long i, j;
	unsigned long sampleCount= BUFFER_SIZE * 10; /*  *10 is to make sure buffer large enough */
	FILE * fp = NULL;
	short * buffers[PS4000_MAX_CHANNEL_BUFFERS];
	PICO_STATUS status;
	unsigned long sampleInterval = 1;
	int index = 0;
	int totalSamples;
	unsigned long triggeredAt = 0;
	short retry = 0;

	for (i = 0; i < unit->ChannelCount; i++) // create data buffers
	{
		buffers[i * 2] = (short*) malloc(sampleCount * sizeof(short));
		buffers[i * 2 + 1] = (short*)malloc(sampleCount * sizeof(short));
		status = ps4000SetDataBuffers(	unit->handle, 
										(PS4000_CHANNEL)i, 
										buffers[i * 2],
										buffers[i * 2 + 1], 
										sampleCount);
	}

	printf("Waiting for trigger...Press a key to abort\n");
	g_autoStop = FALSE;

	
	

	status = ps4000RunStreaming(unit->handle, 
							&sampleInterval, 
							PS4000_US,
							preTrigger, 
							1000000 - preTrigger, 
							TRUE,	//FALSE,
							1000,
							sampleCount);
	


	printf("Streaming data...Press a key to abort\n");

	fopen_s(&fp, "stream.txt", "w");

	if (fp != NULL)
	{
		fprintf(fp,"For each of the %d Channels, results shown are....\n",unit->ChannelCount);
		fprintf(fp,"Maximum Aggregated value ADC Count & mV, Minimum Aggregated value ADC Count & mV\n\n");

		for (i = 0; i < unit->ChannelCount; i++) 
		{
			if (unit->channelSettings[i].enabled) 
				fprintf(fp,"Ch  Max ADC    Max mV  Min ADC    Min mV   ");
		}
		fprintf(fp, "\n");
	}

	totalSamples = 0;
	while (!_kbhit() && !g_autoStop)
	{
		/* Poll until data is received. Until then, GetStreamingLatestValues wont call the callback */
		Sleep(100);
		g_ready = FALSE;

		status = ps4000GetStreamingLatestValues(unit->handle, CallBackStreaming, NULL);
		

		index ++;

		if (g_ready && g_sampleCount > 0) /* can be ready and have no data, if autoStop has fired */
		{
			if (g_trig)
				triggeredAt = totalSamples + g_trigAt;
			totalSamples += g_sampleCount;
			printf("\nCollected %li samples, index = %lu, Total: %d samples  ", g_sampleCount, g_startIndex, totalSamples);

			if (g_trig)
				printf("Trig. at index %lu", triggeredAt);

			if (fp != NULL)
			{
				for (i = g_startIndex; i < (long)(g_startIndex + g_sampleCount); i++) 
				{
					for (j = 0; j < unit->ChannelCount; j++) 
					{
						if (unit->channelSettings[j].enabled) 
						{
							fprintf(	fp,
								"Ch%C %6d = %+6dmV, %6d = %+6dmV   ",
								(char)('A' + j),
								buffers[j * 2][i],
								adc_to_mv(buffers[j * 2][i], unit->channelSettings[PS4000_CHANNEL_A + j].range),
								buffers[j * 2 + 1][i],
								adc_to_mv(buffers[j * 2 + 1][i], unit->channelSettings[PS4000_CHANNEL_A + j].range));
						}
					}
					fprintf(fp, "\n");
				}
			}
			else
				printf("Cannot open the file stream.txt for writing.\n");
		}
	}

	if(fp != NULL) fclose(fp);	

	ps4000Stop(unit->handle);

	if (!g_autoStop) 
	{
		printf("\ndata collection aborted\n");
		_getch();
	}
	for (i = 0; i < unit->ChannelCount * 2; i++) 
	{
		free(buffers[i]);
	}
}

/****************************************************************************
* SetTrigger
*  this function calls the API trigger functions 
****************************************************************************/
PICO_STATUS SetTrigger(	short handle,
												struct tTriggerChannelProperties * channelProperties,
												short nChannelProperties,
												struct tTriggerConditions * triggerConditions,
												short nTriggerConditions,
												TRIGGER_DIRECTIONS * directions,
												struct tPwq * pwq,
												unsigned long delay,
												short auxOutputEnabled,
												long autoTriggerMs)
{
	PICO_STATUS status;

	if ((status = ps4000SetTriggerChannelProperties(handle,
																									channelProperties,
																									nChannelProperties,
																									auxOutputEnabled,
																									autoTriggerMs)) != PICO_OK)
	{

			printf("SetTrigger:ps4000SetTriggerChannelProperties ------ %d \n", status);
			return status;
	}

	if ((status = ps4000SetTriggerChannelConditions(handle,
																									triggerConditions,
																									nTriggerConditions)) != PICO_OK) 
	{
			printf("SetTrigger:ps4000SetTriggerChannelConditions ------ %d \n", status);
			return status;
	}

	if ((status = ps4000SetTriggerChannelDirections(handle,
																									directions->channelA,
																									directions->channelB,
																									directions->channelC,
																									directions->channelD,
																									directions->ext,
																									directions->aux)) != PICO_OK) 
	{
			printf("SetTrigger:ps4000SetTriggerChannelDirections ------ %d \n", status);
			return status;
	}

	if ((status = ps4000SetTriggerDelay(handle, delay)) != PICO_OK) 
	{
		printf("SetTrigger:ps4000SetTriggerDelay ------ %d \n", status);
		return status;
	}

	if((status = ps4000SetPulseWidthQualifier(handle, pwq->conditions, pwq->nConditions, 
																						pwq->direction, pwq->lower, pwq->upper, pwq->type)) != PICO_OK)
	{
		printf("SetTrigger:ps4000SetPulseWidthQualifier ------ %d \n", status);
		return status;
	}
	return status;
}

/****************************************************************************
* CollectBlockImmediate
*  this function demonstrates how to collect a single block of data
*  from the unit (start collecting immediately)
****************************************************************************/
void CollectBlockImmediate(UNIT_MODEL * unit)
{
	struct tPwq pulseWidth;
	struct tTriggerDirections directions;

	memset(&directions, 0, sizeof(struct tTriggerDirections));
	memset(&pulseWidth, 0, sizeof(struct tPwq));

	printf("Collect block immediate...\n");
	printf("Press a key to start\n");
	_getch();

	SetDefaults(unit);

	/* Trigger disabled	*/
	SetTrigger(unit->handle, NULL, 0, NULL, 0, &directions, &pulseWidth, 0, 0, 0);

	BlockDataHandler(unit, "First 10 readings\n", 0);
}

/****************************************************************************
* CollectRapidBlockImmediate
*  this function demonstrates how to rapidly collect a single block of data
*  from the unit (start collecting immediately)
****************************************************************************/
int CollectRapidBlockImmediate(UNIT_MODEL * unit)
{
	struct tPwq pulseWidth;
	struct tTriggerDirections directions;

	memset(&directions, 0, sizeof(struct tTriggerDirections));
	memset(&pulseWidth, 0, sizeof(struct tPwq));

	printf("Collect Rapid block immediate with aggregation...\n");
	printf("Press a key to start\n");
	_getch();

	SetDefaults(unit);

	/* Trigger disabled	*/
	SetTrigger(unit->handle, NULL, 0, NULL, 0, &directions, &pulseWidth, 0, 0, 0);


	return RapidBlockDataHandler(unit, "First 10 readings\n", 0);
}

/****************************************************************************
* CollectRapidBlock_No_Agg
*  this function demonstrates how to use rapid block mode without
*  aggregation
****************************************************************************/
int CollectRapidBlock_No_Agg(UNIT_MODEL * unit)
{
	struct tPwq pulseWidth;
	struct tTriggerDirections directions;

	memset(&directions, 0, sizeof(struct tTriggerDirections));
	memset(&pulseWidth, 0, sizeof(struct tPwq));

	printf("Collect Rapid block immediate without aggregation...\n");
	printf("Press a key to start\n");
	_getch();

	SetDefaults(unit);

	/* Trigger disabled	*/
	SetTrigger(unit->handle, NULL, 0, NULL, 0, &directions, &pulseWidth, 0, 0, 0);

	return No_Agg_RapidBlockDataHandler(unit, "First 10 readings\n", 0);
}


/****************************************************************************
* CollectBlockEts
*  this function demonstrates how to collect a block of
*  data using equivalent time sampling (ETS).
****************************************************************************/
void CollectBlockEts(UNIT_MODEL * unit)
{
	PICO_STATUS status;
	long ets_sampletime;
	short
		triggerVoltage = mv_to_adc(	100, unit->channelSettings[PS4000_CHANNEL_A].range); // ChannelInfo stores ADC counts
	
	struct tTriggerChannelProperties sourceDetails = {triggerVoltage,10,triggerVoltage,10,PS4000_CHANNEL_A,LEVEL };

	struct tTriggerConditions conditions = {	CONDITION_TRUE,
																						CONDITION_DONT_CARE,
																						CONDITION_DONT_CARE,
																						CONDITION_DONT_CARE,
																						CONDITION_DONT_CARE,
																						CONDITION_DONT_CARE,
																						CONDITION_DONT_CARE };
	
	unsigned long delay = 0;
	struct tPwq pulseWidth;
	struct tTriggerDirections directions;

	memset(&pulseWidth, 0, sizeof(struct tPwq));
	memset(&directions, 0, sizeof(struct tTriggerDirections));
	directions.channelA = RISING;

	printf("Collect ETS block...\n");
	printf("Collects when value rises past %d", scaleVoltages? 
		adc_to_mv(sourceDetails.thresholdUpper,	unit->channelSettings[PS4000_CHANNEL_A].range)	// If scaleVoltages, print mV value
		: sourceDetails.thresholdUpper);																// else print ADC Count
	printf(scaleVoltages? "mV\n" : "ADC Counts\n");
	printf("Press a key to start...\n");
	_getch();

	SetDefaults(unit);

	/* Trigger enabled
	* Rising edge
	* Threshold = 1500mV
	* 10% pre-trigger  (negative is pre-, positive is post-) */

	status = SetTrigger(unit->handle, &sourceDetails, 1, &conditions, 1,
		&directions, &pulseWidth, delay, 0, 0);

	/* printf("Set Trigger : %x" , status); */

	/* Enable ETS in fast mode, the computer will store 100 cycles but interleave only 10 */
	status = ps4000SetEts(unit->handle, PS4000_ETS_FAST, 20, 4, &ets_sampletime);

	/*printf("Set ETS : %x" , status);*/
	printf("ETS Sample Time is: %ld\n", ets_sampletime);

	BlockDataHandler(unit, "Ten readings after trigger\n", BUFFER_SIZE / 10 - 5); // 10% of data is pre-trigger
}

/****************************************************************************
* CollectBlockTriggered
*  this function demonstrates how to collect a single block of data from the
*  unit, when a trigger event occurs.
****************************************************************************/
void CollectBlockTriggered(UNIT_MODEL * unit)
{
	short	triggerVoltage = mv_to_adc(1000,	unit->channelSettings[PS4000_CHANNEL_A].range); // ChannelInfo stores ADC counts

	struct tTriggerChannelProperties sourceDetails = {	triggerVoltage,
																											triggerVoltage,
																											256 * 10,
																											PS4000_CHANNEL_A,
																											LEVEL };

	struct tTriggerConditions conditions = {	CONDITION_TRUE,
																						CONDITION_DONT_CARE,
																						CONDITION_DONT_CARE,
																						CONDITION_DONT_CARE,
																						CONDITION_DONT_CARE,
																						CONDITION_DONT_CARE,
																						CONDITION_DONT_CARE };

	struct tPwq pulseWidth;
	
	struct tTriggerDirections directions = {	RISING,
																						NONE,
																						NONE,
																						NONE,
																						NONE,
																						NONE };
	
	memset(&pulseWidth, 0, sizeof(struct tPwq));

		printf("Collect block triggered...\n");
	printf("Collects when value rises past %d", scaleVoltages?
		adc_to_mv(sourceDetails.thresholdUpper, unit->channelSettings[PS4000_CHANNEL_A].range)	// If scaleVoltages, print mV value
		: sourceDetails.thresholdUpper);																// else print ADC Count
	printf(scaleVoltages?"mV\n" : "ADC Counts\n");

	printf("Press a key to start...\n");
	_getch();

	SetDefaults(unit);

	/* Trigger enabled
	* Rising edge
	* Threshold = 100mV */
	SetTrigger(unit->handle, &sourceDetails, 1, &conditions, 1, &directions, &pulseWidth, 0, 0, 0);

	BlockDataHandler(unit, "Ten readings after trigger\n", 0);
}

/****************************************************************************
* Initialise unit' structure with Variant specific defaults
****************************************************************************/
void get_info(UNIT_MODEL * unit)
{
	char description [6][25]= { "Driver Version",
															"USB Version",
															"Hardware Version",
															"Variant Info",
															"Serial",
															"Error Code" };
	short i, r = 0;
	char line [80];
	int variant;
	PICO_STATUS status = PICO_OK;

	if (unit->handle) 
	{
		for (i = 0; i < 5; i++) 
		{
			status = ps4000GetUnitInfo(unit->handle, line, sizeof(line), &r, i);
			if (i == 3) 
			{
				variant = atoi(line);
			}
			printf("%s: %s\n", description[i], line);
		}

	switch (variant) 
	{
	case MODEL_PS4223:
		unit->model           = MODEL_PS4223;
		unit->signalGenerator = FALSE;
		unit->ETS             = FALSE;
		unit->firstRange      = PS4000_50MV;
		unit->lastRange       = PS4000_50V;
		unit->ChannelCount    = DUAL_SCOPE;
		break;

	case MODEL_PS4224:
		unit->model           = MODEL_PS4224;
		unit->signalGenerator = FALSE;
		unit->ETS             = FALSE;
		unit->firstRange      = PS4000_50MV;
		unit->lastRange       = PS4000_20V;
		unit->ChannelCount    = DUAL_SCOPE;
		break;

	case MODEL_PS4423:
		unit->model           = MODEL_PS4423;
		unit->signalGenerator = FALSE;
		unit->ETS             = FALSE;
		unit->firstRange      = PS4000_50MV;
		unit->lastRange       = PS4000_50V;
		unit->ChannelCount    = QUAD_SCOPE;
		break;

	case MODEL_PS4424:
		unit->model           = MODEL_PS4424;
		unit->signalGenerator = FALSE;
		unit->ETS             = FALSE;
		unit->firstRange      = PS4000_50MV;
		unit->lastRange       = PS4000_20V;
		unit->ChannelCount    = QUAD_SCOPE;
		break;

	case MODEL_PS4226:
		unit->model           = MODEL_PS4226;
		unit->signalGenerator = TRUE;
		unit->ETS             = TRUE;
		unit->firstRange      = PS4000_50MV;
		unit->lastRange       = PS4000_20V;
		unit->ChannelCount    = DUAL_SCOPE;
		break;

	case MODEL_PS4227:
		unit->model           = MODEL_PS4227;
		unit->signalGenerator = TRUE;
		unit->ETS             = TRUE;
		unit->firstRange      = PS4000_50MV;
		unit->lastRange       = PS4000_20V;
		unit->ChannelCount    = DUAL_SCOPE;
		break;

	case MODEL_PS4262:
		unit->model           = MODEL_PS4262;
		unit->signalGenerator = TRUE;
		unit->ETS             = FALSE;
		unit->firstRange      = PS4000_10MV;
		unit->lastRange       = PS4000_20V;
		unit->ChannelCount    = DUAL_SCOPE;
		break;

	default:
		break;
		}
	}
}

/****************************************************************************
* Select input voltage ranges for channels A and B
****************************************************************************/
void set_voltages(UNIT_MODEL * unit)
{
	int i, ch;
	int count = 0;

	/* See what ranges are available... */
	for (i = unit->firstRange; i <= unit->lastRange; i++) 
	{
		printf("%d -> %d mV\n", i, inputRanges[i]);
	}

	do
	{
		/* Ask the user to select a range */
		printf("Specify voltage range (%d..%d)\n", unit->firstRange, unit->lastRange);
		printf("99 - switches channel off\n");
		for (ch = 0; ch < unit->ChannelCount; ch++) 
		{
			printf("\n");
			do 
			{
				printf("Channel %c: ", 'A' + ch);
				fflush(stdin);
				scanf_s("%hd", &unit->channelSettings[ch].range);
			} while (unit->channelSettings[ch].range != 99 && (unit->channelSettings[ch].range < unit->firstRange || unit->channelSettings[ch].range > unit->lastRange));

			if (unit->channelSettings[ch].range != 99) 
			{
				printf(" - %d mV\n", inputRanges[unit->channelSettings[ch].range]);
				unit->channelSettings[ch].enabled = TRUE;
				count++;
			} 
			else 
			{
				printf("Channel Switched off\n");
				unit->channelSettings[ch].enabled = FALSE;
			}
		}
		printf(count == 0? "\n** At least 1 channel must be enabled **\n\n":"");
	}
	while(count == 0);	// must have at least one channel enabled

	SetDefaults(unit);	// Put these changes into effect
}

/****************************************************************************
*
* Select timebase, set oversample to on and time units as nano seconds
*
****************************************************************************/
void SetTimebase(UNIT_MODEL unit)
{
	long timeInterval;
	long maxSamples;

	printf("Specify desired timebase: ");
	fflush(stdin);
	scanf_s("%lud", &timebase);

	while (ps4000GetTimebase(unit.handle, timebase, BUFFER_SIZE, &timeInterval, 1, &maxSamples, 0))
	{
		timebase++;  // Increase timebase if the one specified can't be used. 
	}

	printf("Timebase used %lu = %ldns Sample Interval\n", timebase, timeInterval);
	oversample = TRUE;
}


/****************************************************************************
* Sets the signal generator
* - allows user to set frequency and waveform
* - allows for custom waveforms...
* - PS4226 & 4227 - values 0..4095, up to 8192 samples long
* - PS4262 - values -32768..32767, 4096 samples long
******************************************************************************/
void SetSignalGenerator(UNIT_MODEL unit)
{
	PICO_STATUS status;
	short waveform;
	int frequency = 1;
	char fileName [128];
	FILE * fp = NULL;
	short *arbitraryWaveform;
	long waveformSize = 0;
	unsigned long pkpk = 1000000;
	long offset = 0;
	char ch;
	short choice;
	double delta;
	short AWGFileSize;
	float UCVal;
	int maxFreq;


	while (kbhit())			// use up keypress
		getch();

	do
	{
		printf("\nSignal Generator\n================\n");
		printf("0 - SINE         1 - SQUARE\n");
		printf("2 - TRIANGLE     3 - DC VOLTAGE\n");
		printf("4 - RAMP UP      5 - RAMP DOWN\n");
		printf("6 - SINC         7 - GAUSSIAN\n");
		printf("8 - HALF SINE    A - AWG WAVEFORM\n");
		printf("F - SigGen Off\n\n");

		ch = getch();

		if (ch >= '0' && ch <='9')
			choice = ch -'0';
		else
			ch = toupper(ch);
	}
	while(ch != 'A' && ch != 'F' && (ch < '0' || ch > '8')  );


	if (unit.model == MODEL_PS4262)									
	{
		AWGFileSize = MAX_SIG_GEN_BUFFER_SIZE>>1;						// PS4262 has 4K buffer
		UCVal = 5.2083e-6;																	// 1 / 192KHz
		maxFreq = 20000;																		// 20MHz maximum frequency							
	}
	else																						
	{
		AWGFileSize = MAX_SIG_GEN_BUFFER_SIZE;							// PS4226 / 4227 has 8K buffer
			
		UCVal = 5e-8;																				// 1/20MHz
		maxFreq = 100000;																		// 100MHz maximum frequency		
	}



	if(ch == 'F')				// If we're going to turn off siggen
	{
		printf("Signal generator Off\n");
		waveform = 8;		// DC Voltage
		pkpk = 0;				// 0V
		waveformSize = 0;
	}
	else
	if (ch == 'A')		// Set the AWG
	{
		arbitraryWaveform = (short*)malloc( AWGFileSize * sizeof(short));
		memset(arbitraryWaveform, 0, AWGFileSize * sizeof(short));

		waveformSize = 0;

		printf("Select a waveform file to load: ");
		scanf_s("%s", fileName, 128);
		if (fopen_s(&fp, fileName, "r") == 0) 
		{ // Having opened file, read in data - one number per line 
			while (EOF != fscanf_s(fp, "%hi", (arbitraryWaveform + waveformSize))&& waveformSize++ < AWGFileSize - 1);
			fclose(fp);
			printf("File successfully loaded\n");
		} 
		else 
		{
			printf("Invalid filename\n");
			return;
		}
	}
	else			// Set one of the built in waveforms
	{
		switch (choice)
		{
			case 0:
				waveform = PS4000_SINE;
				break;

			case 1:
				waveform = PS4000_SQUARE;
				break;

			case 2:
				waveform = PS4000_TRIANGLE;
				break;

			case 3:
				waveform = PS4000_DC_VOLTAGE;
				do 
				{
					printf("\nEnter offset in uV: (0 to 2500000)\n"); // Ask user to enter DC offset level;
					scanf_s("%lu", &offset);
				} while (offset < 0 || offset > 10000000);
				break;

			case 4:
				waveform = PS4000_RAMP_UP;
				break;

			case 5:
				waveform = PS4000_RAMP_DOWN;
				break;

			case 6:
				waveform = PS4000_SINC;
				break;

			case 7:
				waveform = PS4000_GAUSSIAN;
				break;

			case 8:
				waveform = PS4000_HALF_SINE;
				break;

			default:
				waveform = PS4000_SINE;
				break;
		}
	}

	if(waveform < 8 || ch == 'A' )				// Find out frequency if required
	{
		do 
		{
			printf("\nEnter frequency in Hz: (1 to %d)\n", maxFreq); // Ask user to enter signal frequency;
			scanf_s("%lu", &frequency);
		} while (frequency <= 0 || frequency > maxFreq);
	}

	if (waveformSize > 0)		
	{
		delta = ((1.0 * frequency * waveformSize) / AWGFileSize) * (4294967296.0 * UCVal);		
	
		status = ps4000SetSigGenArbitrary(	unit.handle, 
																				0,												// offset voltage
																				pkpk,											// PkToPk in microvolts. Max = 4uV  +2v to -2V
																				(unsigned long)delta,			// start delta
																				(unsigned long)delta,			// stop delta
																				0, 
																				0, 
																				arbitraryWaveform, 
																				waveformSize, 
																				(SWEEP_TYPE)0, 
																				0, 
																				SINGLE, 
																				0, 
																				0, 
																				SIGGEN_RISING,
																				SIGGEN_NONE, 
																				0);

		printf(status?"\nps4000SetSigGenArbitrary: Status Error 0x%x \n":"", (unsigned int)status);		// If status != 0, show the error
	} 
	else 
	{
		status = ps4000SetSigGenBuiltIn(unit.handle, 
																		offset, 
																		pkpk, 
																		waveform, 
																		(float)frequency, 
																		(float)frequency, 
																		0, 
																		0, 
																		(SWEEP_TYPE)0, 
																		0, 
																		0, 
																		0, 
																		(SIGGEN_TRIG_TYPE)0, 
																		(SIGGEN_TRIG_SOURCE)0, 
																		0);
		printf(status?"\nps4000SetSigGenBuiltIn: Status Error 0x%x \n":"", (unsigned int)status);		// If status != 0, show the error
	}
}



/****************************************************************************
* CollectStreamingImmediate
*  this function demonstrates how to collect a stream of data
*  from the unit (start collecting immediately)
***************************************************************************/
void CollectStreamingImmediate(UNIT_MODEL * unit)
{
	struct tPwq pulseWidth;
	struct tTriggerDirections directions;

	memset(&pulseWidth, 0, sizeof(struct tPwq));
	memset(&directions, 0, sizeof(struct tTriggerDirections));

	SetDefaults(unit);

	printf("Collect streaming...\n");
	printf("Data is written to disk file (stream.txt)\n");
	printf("Press a key to start\n");
	_getch();


	/* Trigger disabled	*/
	SetTrigger(unit->handle, NULL, 0, NULL, 0, &directions, &pulseWidth, 0, 0, 0);

	StreamDataHandler(unit, 0);
}

/****************************************************************************
* CollectStreamingTriggered
*  this function demonstrates how to collect a stream of data
*  from the unit (start collecting on trigger)
***************************************************************************/
void CollectStreamingTriggered(UNIT_MODEL * unit)
{
	short	triggerVoltage = mv_to_adc(1000, unit->channelSettings[PS4000_CHANNEL_A].range); // ChannelInfo stores ADC counts

	struct tTriggerChannelProperties sourceDetails = {	triggerVoltage,
																											triggerVoltage,
																											256 * 10,
																											PS4000_CHANNEL_A,
																											LEVEL };

	struct tTriggerConditions conditions = {	CONDITION_TRUE,
																						CONDITION_DONT_CARE,
																						CONDITION_DONT_CARE,
																						CONDITION_DONT_CARE,
																						CONDITION_DONT_CARE,
																						CONDITION_DONT_CARE,
																						CONDITION_DONT_CARE };

	struct tPwq pulseWidth;
	
	struct tTriggerDirections directions = {	RISING,
																						NONE,
																						NONE,
																						NONE,
																						NONE,
																						NONE };			
	
	memset(&pulseWidth, 0, sizeof(struct tPwq));

	printf("Collect streaming triggered...\n");
	printf("Data is written to disk file (data.txt)\n");
	printf("Press a key to start\n");
	_getch();
	SetDefaults(unit);

	/* Trigger enabled
	* Rising edge
	* Threshold = 100mV */

	SetTrigger(unit->handle, &sourceDetails, 1, &conditions, 1, &directions, &pulseWidth, 0, 0, 0);
	
	StreamDataHandler(unit, 100000);
}


/****************************************************************************
* DisplaySettings 
* Displays information about the user configurable settings in this example
* Parameters 
* - unit        pointer to the UNIT structure
*
* Returns       none
***************************************************************************/
void DisplaySettings(UNIT_MODEL *unit)
{
	int ch;
	int voltage;

	printf("\n\nReadings will be scaled in (%s)\n", (scaleVoltages)? ("mV") : ("ADC counts"));

	for (ch = 0; ch < unit->ChannelCount; ch++)
	{
		voltage = inputRanges[unit->channelSettings[ch].range];
		printf("Channel %c Voltage Range = ", 'A' + ch);

		if (voltage == 0)
			printf("Off\n");
		else
		{
			if (voltage < 1000)
				printf("%dmV\n", voltage);
			else
				printf("%dV\n", voltage / 1000);
		}
	}
	printf("\n");
}

int main(void)
{
	char ch;
	int i;
	PICO_STATUS status;
	UNIT_MODEL unit;
	struct tPwq pulseWidth;
	struct tTriggerDirections directions;

	printf("PS4000 driver example program\n");
	printf("\n\nOpening the device...\n");

	status = ps4000OpenUnit(&(unit.handle));
	printf("Handle: %d\n", unit.handle);
	
	if (status != PICO_OK && status != PICO_EEPROM_CORRUPT) 
	{
		printf("Unable to open device\n");
		printf("Error code : %d\n", (int)status);
		while (!_kbhit());
		exit( 99); // exit program - nothing after this executes
	}

	printf("Device opened successfully, cycle %d\n\n", ++cycles);

	// setup devices
	get_info(&unit);
	timebase = 1;

	for (i = 0; i < MAX_CHANNELS; i++) 
	{
		unit.channelSettings[i].enabled = TRUE;
		unit.channelSettings[i].DCcoupled = TRUE;
		unit.channelSettings[i].range = PS4000_5V;
	}

	memset(&directions, 0, sizeof(struct tTriggerDirections));
	memset(&pulseWidth, 0, sizeof(struct tPwq));

	SetDefaults(&unit);

	/* Trigger disabled	*/
	SetTrigger(unit.handle, NULL, 0, NULL, 0, &directions, &pulseWidth, 0, 0, 0);

	ch = ' ';
	while (ch != 'X')
	{
		DisplaySettings(&unit);

		printf("\n");
		printf("B - immediate block                             V - Set voltages\n");
		printf("T - triggered block                             I - Set timebase\n");
		printf("R - immdeiate rapid block with aggreation       A - ADC counts/mV\n");
		printf("N - rapid block without aggreation\n");
		printf("Q - collect a block using ETS\n");
		printf("S - immediate streaming\n");
		printf("W - triggered streaming\n");
		printf("G - Signal generator\n");
		printf("                                                X - exit\n");
		printf("Operation:");

		ch = toupper(_getch());

		printf("\n\n");
		switch (ch) 
		{
		case 'B':
			CollectBlockImmediate(&unit);
			break;

		case 'R':
			CollectRapidBlockImmediate(&unit);
			break;

		case 'N':
			CollectRapidBlock_No_Agg(&unit);
			break;

		case 'T':
			CollectBlockTriggered(&unit);
			break;

		case 'S':
			CollectStreamingImmediate(&unit);
			break;

		case 'W':
			CollectStreamingTriggered(&unit);
			break;

		case 'Q':
			if(unit.ETS == FALSE)
			{
				printf("This model does not have ETS\n\n");
				break;
			}

			CollectBlockEts(&unit);
			break;

		case 'G':
			if(unit.signalGenerator == FALSE)
			{
				printf("This model does not have a signal generator\n\n");
				break;
			}

			SetSignalGenerator(unit);
			break;

		case 'V':
			set_voltages(&unit);
			break;

		case 'I':
			SetTimebase(unit);
			break;

		case 'A':
			scaleVoltages = !scaleVoltages;
			break;

		case 'X':
			break;

		default:
			printf("Invalid operation\n");
			break;
		}
	}
	
	ps4000CloseUnit(unit.handle);
	return 1;
}
